document.addEventListener("DOMContentLoaded", function() {

    let windowHeight = window.innerHeight,
        ngPanel = document.getElementById("ng-panel-1"),
        ngTranslate = document.querySelector("#ng-panel-1 .ng-panel__translate"),
        ngZoom = document.querySelector("#ng-panel-1 .ng-panel__zoom"),
        ngLayers = document.querySelectorAll("#ng-panel-1 .ng-panel__layer"),
        ngAnchors = document.querySelectorAll("[data-anchor]"),
        firstLayerId = ngAnchors[0].getAttribute("data-anchor"),
        currentLayerId = firstLayerId,
        introBlock = document.querySelector(".content-group.intro"),
        parallaxIntro = document.querySelector(".content-group.parallax"),
        mainContentBlock = document.getElementById("mainContent");

    let parallaxX0, parallaxY0, parallaxX1, parallaxY1, parallaxW0, parallaxH0, parallaxW1, parallaxH1;

    let firstLayer = document.getElementById(firstLayerId);

    if (parallaxIntro !== null) {
        parallaxX0 = parallaxIntro.getAttribute("data-parallax-x0"),
        parallaxY0 = parallaxIntro.getAttribute("data-parallax-y0"),
        parallaxX1 = parallaxIntro.getAttribute("data-parallax-x1"),
        parallaxY1 = parallaxIntro.getAttribute("data-parallax-y1"),
        parallaxW0 = parallaxIntro.getAttribute("data-parallax-w0"),
        parallaxH0 = parallaxIntro.getAttribute("data-parallax-h0"),
        parallaxW1 = parallaxIntro.getAttribute("data-parallax-w1"),
        parallaxH1 = parallaxIntro.getAttribute("data-parallax-h1");
    }

    var anchorTags = document.querySelectorAll(".ng-nav__item a");

    anchorTags.forEach(function(anchor) {
        anchor.addEventListener('click', function() {
            const desktopDevice = window.matchMedia('(min-width: 992px)');
            let deviceType = (desktopDevice.matches) ? "desktop" : "mobile";
            let layerHref = anchor.getAttribute('href');
            let layerId = layerHref.replace(/#/g, "slide-");
            let layer = document.getElementById(layerId);
            console.log(layer);
            goToLayer(deviceType, ngTranslate, ngZoom, layer, ngPanel);
        });
    })

    // every time window is scrolled, check anchors positions
    window.addEventListener('scroll', function() {

        const desktopDevice = window.matchMedia('(min-width: 992px)');
        let deviceType = (desktopDevice.matches) ? "desktop" : "mobile";

        // parallax function

        if (parallaxIntro !== null) {
            let parallaxTop = parallaxIntro.getBoundingClientRect().top;

            if (parallaxTop < 0) {

                let bottomDistance = parallaxIntro.offsetHeight + parallaxTop;

                if (bottomDistance > 0) {

                    if (!ngPanel.classList.contains("parallax")) {
                        ngPanel.classList.add("parallax");
                        currentLayerId = parallaxIntro.getAttribute('id');
                    }

                    let transLateRatio = scale(bottomDistance, 0, parallaxIntro.offsetHeight, 1, 0);

                    let translateX = (parallaxX1 == parallaxX0) ? parallaxX1 : parseInt(parallaxX0) + (transLateRatio * (parallaxX1 - parallaxX0)),
                        translateY = (parallaxY1 == parallaxY0) ? parallaxY1 : parseInt(parallaxY0) + (transLateRatio * (parallaxY1 - parallaxY0)),
                        targetWidth = (parallaxW1 == parallaxW0) ? parallaxW1 : parseInt(parallaxW0) + (transLateRatio * (parallaxW1 - parallaxW0)),
                        targetHeight = (parallaxH1 == parallaxH0) ? parallaxH1 : parseInt(parallaxH0) + (transLateRatio * (parallaxH1 - parallaxH0));

                    moveCamera(ngTranslate, ngZoom, translateX, translateY, targetWidth, targetHeight, "contain", ngPanel);

                } else {
                    if (ngPanel.classList.contains("parallax")) {
                        ngPanel.classList.remove("parallax");
                    }
                }

            } else {
                if (ngPanel.classList.contains("parallax")) {
                    ngPanel.classList.remove("parallax");
                }
            }
        }

        currentLayerId = checkAnchors(
            deviceType,
            ngAnchors,
            windowHeight,
            currentLayerId,
            ngPanel,
            ngTranslate,
            ngZoom
        );

        if (introBlock.getBoundingClientRect().top + introBlock.offsetHeight - 300 < 0) {
            if (!mainContentBlock.classList.contains("overlay-hidden")) {
                mainContentBlock.classList.add("overlay-hidden");
            }
        } else {
            if (mainContentBlock.classList.contains("overlay-hidden")) {
                mainContentBlock.classList.remove("overlay-hidden");
            }
        };

    });

    if(ngAnchors[0] !== null) {

        const desktopDevice = window.matchMedia('(min-width: 992px)');
        let deviceType = (desktopDevice.matches) ? "desktop" : "mobile";

        // if first anchor matches a layer, navigate to layer
        goToLayer(deviceType, ngTranslate, ngZoom, firstLayer, ngPanel);
    }

});

function checkAnchors(
    device,
    ngAnchors,
    windowHeight,
    currentLayerId,
    ngPanel,
    ngTranslate,
    ngZoom
) {
    // iterate through all registered anchors
    ngAnchors.forEach(function(anchor, i, anchors) {

        // calculate distance between top of anchor and middle of screen
        let distanceToFocus = anchor.getBoundingClientRect().top - (windowHeight / 4 * 3);

        // if distance is within threshold, assign anchor as currentAnchor
        if (distanceToFocus < 200 && distanceToFocus > -200) {

            let layerId = anchor.getAttribute('data-anchor');

            if(layerId !== currentLayerId) {
                currentLayerId = layerId;
                // find matching layer
                let targetLayer = document.getElementById(layerId);
                // if layer exists, navigate to layer
                if (targetLayer !== null) {
                    goToLayer(device, ngTranslate, ngZoom, targetLayer, ngPanel);
                } else {
                    console.log("layer nao existe")
                }

            }
        }

    });

    return currentLayerId;

}

function scale (number, inMin, inMax, outMin, outMax) {
    return (number - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

function moveCamera(
    ngTranslate,
    ngZoom,
    targetX,
    targetY,
    targetWidth,
    targetHeight,
    targetFit,
    ngPanel
) {

    // get stage properties
    let availableWidth = ngPanel.offsetWidth,
        availableHeight = ngPanel.offsetHeight;

    // calculate target scale (zoom), according to target layer and stage sizes
    let targetHeightScale = availableHeight / targetHeight,
        targetWidthScale = availableWidth / targetWidth,
        targetScale;
    if (targetFit == "cover") {
        targetScale = (targetHeightScale < targetWidthScale) ? targetWidthScale : targetHeightScale;
    } else if (targetFit == "contain") {
        targetScale = (targetHeightScale < targetWidthScale) ? targetHeightScale : targetWidthScale;
    } else {
        targetScale = (targetHeightScale < targetWidthScale) ? targetHeightScale : targetWidthScale;
    }

    // transform stage to focus on target layer
    ngTranslate.style.transform = `translateZ(0) translate(-${targetX}px, -${targetY}px)`;
    ngZoom.style.transform = `translateZ(0) translate(0, 0) scale(${targetScale})`;

    // ngTranslate.style.webkitTransform = `translateZ(0) translate(-${targetX}px, -${targetY}px)`;
    // ngZoom.style.webkitTransform = `translateZ(0) translate(0, 0) scale(${targetScale})`;

}

function goToLayer(viewPort, ngTranslate, ngZoom, targetLayer, ngPanel) {

    console.log(targetLayer);

    // get target layer properties
    let targetX = targetLayer.getAttribute(`data-x-${ viewPort }`),
        targetY = targetLayer.getAttribute(`data-y-${ viewPort }`),
        targetHeight = targetLayer.getAttribute(`data-h-${ viewPort }`),
        targetWidth = targetLayer.getAttribute(`data-w-${ viewPort }`),
        targetFit = targetLayer.getAttribute(`data-fit`);

    moveCamera(ngTranslate, ngZoom, targetX, targetY, targetWidth, targetHeight, targetFit, ngPanel);

    // update css classes
    let lastLayer = document.querySelector("#ng-panel-1 .ng-panel__layer.active");
    if (lastLayer !== null) {
        lastLayer.classList.remove("active");
    }
    targetLayer.classList.add("active");

    let triggerId = targetLayer.getAttribute("id").replace("slide-", "");

    let boundAnimation = document.querySelector(`.event-animation[data-trigger="#${triggerId}"]`);

    if (boundAnimation !== null) {

        if (!boundAnimation.classList.contains("active")) {
            boundAnimation.classList.add("active");
        }

    }

}
